#Perform cross validation for linear model
crossValidateLM = function(kernelData, nfolds, cvType, fitFun, testData, xNames = NULL, finalFitFull = FALSE, ...,
                           testDataNone, transFun = meanCenterVec, outVar = "a", returnFullFit = FALSE, cv = TRUE, centerMoran = TRUE){
    fitFun = match.fun(fitFun)
    #Create folds
    folds = makeFolds(nfolds, kernelData, cvType)
    uniqueFolds = seq_len(nfolds); names(uniqueFolds) = uniqueFolds
    #Fit model on training data
    if(is.null(xNames))
        xNames = xNames = grep(names(kernelData), pattern = "b.", value = TRUE)
    weightMat = 1/as.matrix(dist(kernelData[, c("x", "y")])); diag(weightMat) = 0 #Weight matrix for Moran's Is

    samId = folds != sample(uniqueFolds, 1)
    samData = if(finalFitFull) kernelData else kernelData[samId,]
    fullFitFold = fitFun(samData, xNames = xNames, outVar = outVar, ...)#Also omit one fold here for training
    fullFit = fitFun(kernelData, xNames = xNames, outVar = outVar, ...)
    fullCoef = getCoef(fullFitFold)[-1]#Exclude intercept
    if(cv){
        trainFitsR2C2 = sapply(uniqueFolds, function(uf){
            trainId = !(folds %in% uf)
            trainFit = fitFun(kernelData[trainId,], xNames = xNames, outVar = outVar, ...)
            testPreds = transFun(predictWrapper(trainFit, kernelData[!trainId,xNames, drop = FALSE]))#transFun()
            testDat  = transFun(kernelData[[outVar]][!trainId])
            R2 = 1-mean((testDat-testPreds)^2)/varN(testDat)
            moransI = moransIobs(testDat - testPreds, W = weightMat[!trainId, !trainId])
            if(centerMoran)
                moransI = moransI + 1/(length(testPreds)-1)
            return(c("R2" = R2, "moransI" = moransI))
        })
        meanR2 = mean(trainFitsR2C2[1,]); meanMoransI = mean(trainFitsR2C2[2,])
        moransItrain = moransIobs(x = c(transFun(kernelData[[outVar]]) - transFun(predictWrapper(fullFit, newData = kernelData[,xNames, drop =FALSE]))), W = weightMat) #Find Moran's I on residuals
        if(centerMoran)
            moransItrain = moransItrain + 1/(nrow(kernelData)-1)
    } else {meanR2= moransItrain= NA}
    # A test dataset, as large as the training datasets
    if(returnFullFit){
        return(list("cvR2" = meanR2, "moransIresid" = moransItrain, "fit" = fullFit))
    }
    predTrain = transFun(predictWrapper(fullFit, newData = kernelData[,xNames]))
    trainR2 = calcR2(kernelData[[outVar]], predTrain, transFun)
    MSEParam = mean((fullCoef-attr(kernelData, "betas"))^2)
    predNew = transFun(predictWrapper(fullFitFold, newData = testData[,xNames]))
    testR2 = calcR2(testData[[outVar]], predNew, transFun)
    testMoransI = moransIobsLoc(transFun(testData[[outVar]]) - predNew, testData[, c("x", "y")])
    if(centerMoran)
        testMoransI = testMoransI + 1/(length(predNew)-1)
    predNewNone = transFun(predictWrapper(fullFitFold, newData = testDataNone[,xNames]))
    testR2none = calcR2(testDataNone[[outVar]], predNewNone, transFun)
    return(c("cvR2" = meanR2,  "testR2spatial" = testR2, "testR2none" = testR2none, "trainR2" = trainR2, "MSEParam" = MSEParam, "moransIresid" = moransItrain, "cvMoransI" = meanMoransI, "testMoransI" = testMoransI))
}
predictWrapper = function(fit, newData){
    if(is.numeric(fit)){
        cbind(1,as.matrix(newData)) %*% fit
    } else if (class(fit)[1] %in% c("esf", "resf")){
        cbind(1,as.matrix(newData)) %*% fit$b[,"Estimate"]
    } else if(class(fit)[1]=="gam") {
        predict(fit, newdata = newData, type = "response",
                exclude = "te(x,y)", newdata.guaranteed = TRUE) #Exclude spline components for gam for prediction
    } else if (class(fit)[1] == "Sarlm"){
        predict(fit, newdata = newData, pred.type = "TS")[,1]
    }else if (class(fit)=="summary.inla"){
        cbind(1,as.matrix(newData)) %*% fit$fixed[, "mean"]
    }else {
        stats:::predict(fit, newdata = newData, type = "response")
    }
}
meanCenterA = function(df){
    df$a = df$a-mean(df$a)
    df
}
meanCenterVec = function(x) x-mean(x)
varN = function(x) mean((x-mean(x))^2)
calcR2 = function(obs, pred, transFun){
    1-mean((transFun(obs)-pred)^2)/varN(obs)
}

