#Kernel generation function
drawKernel = function(depType, gridSize, sd = sqrt(gridSize), kernelMult = gridSize,
                      grid = NULL, unifBounds = c(-1, 1)*0.5, sdBetaEdge = sqrt(50),
                      xyBetas = NULL, maxGrid = gridSize){
    if(is.null(grid)){
        grid = expand.grid("x" = seq_len(gridSize), "y" = seq_len(gridSize))
    }
     if(grepl(depType, pattern= "kernel")){
        aCenter = runif(2, min(grid), max(grid))
        aDist = dist2point(grid, aCenter)
        dnorm(aDist, sd = sd)*kernelMult
    } else if(grepl(depType, pattern= "linear")){
        if(is.null(xyBetas))
            xyBetas = runif(2, unifBounds[1], unifBounds[2])
        c(as.matrix(grid) %*% xyBetas)
    } else if (grepl(depType, pattern= "edge")){
        betaEdge = rnorm(1, sd = sdBetaEdge)
        dist2Edge = apply(grid, 1, function(xy) min(abs(c(xy, xy-(maxGrid+1)))))
        1/dist2Edge*betaEdge
    } else if(any(sapply(c("none", "gaussCor", "spherCor", "linCor"), grepl, depType))){
        integer(nrow(grid))
    } else {stop("Dependence kernel unknown!")}
}
