#Fit a least squares model
fitLSmodel = function(kernelData, soCor, corStruct, xNames,
                      optControl, kSpline = 5, outVar = "a"){
    finalForm = formula(paste(outVar, "~", paste(xNames, collapse = "+")))
    switch(soCor,
           "None" = lm(finalForm, data = kernelData)$coef,
           "Marginal"= {
               margCorMat = getMargCorMat(kernelData = kernelData, glsSt = glsStruct(corStruct),
                                          control = optControl, outVar =outVar)$corMat
               desMat = margCorMat %*% cbind(1, as.matrix(kernelData[, c(outVar, xNames)]))
               lm.fit(y = desMat[,2], x = desMat[,-2])$coef
           },
           "Conditional" = do.call(nlme::gls, args = list("model"= finalForm, "data" = kernelData,
                                     "correlation" = corStruct, "control" = optControl)),#do.call to avoid problems with predict.gls
           "GAM" = gam(update(finalForm, "~. + te(x,y, k = kSpline, bs = c('tp','tp'))"), data = kernelData),
           "sf"  = getSFfitspmoran(kernelData, regs = xNames, outVar = outVar),
           "sfRandom" = getSFfitspmoran(kernelData, regs = xNames, random = TRUE, outVar = outVar),
           "sar" = fitSAR(kernelData, regs = xNames, outVar = outVar),
           "inlaAR1" = fitInla(kernelData, method = "AR1", regs = xNames, outVar = outVar),
           "inlaMatern" = fitInla(kernelData, method = "Matern", regs = xNames, outVar = outVar)
    )
}
#Calculate inverse square root
getInvSqrt = function(mat){
    ei <- eigen(mat)
    V <- ei$vectors
    V %*% diag(1 / sqrt(ei$values)) %*% t(V)
}



