#Fit a penalised least squares model
fitPLSmodel = function(kernelData, nfolds, soCor, corStruct, optControl, cvType, xNames = NULL, outVar = "a",
                       alpha = 0.5, keep = FALSE, lambdas, transFun = meanCenterVec, bPattern = "b.", finalFitFull = FALSE,...){
    if(is.null(xNames))
        xNames = grep(names(kernelData), pattern = bPattern, value = TRUE)
    glsSt = glsStruct(corStruct)
    initialFit = fitPLSsingle(x = as.matrix(kernelData[,xNames]), y = kernelData[, outVar], cv = TRUE,
                            alpha = alpha, keep = keep, soCor = if(soCor %in% c("GAM", "Conditional")) "None" else soCor,
                            lambda = NULL, xNames = xNames, kernelData = kernelData, glsSt = glsSt, outVar = outVar,
                            optControl = optControl, nlambda = if(soCor %in% c("GAM", "Conditional")) 20 else 100)
    lambdas = initialFit$lambda #The lambda sequence
    folds = makeFolds(nfolds, kernelData, cvType)
    weightMat = 1/as.matrix(dist(kernelData[, c("x", "y")])); diag(weightMat) = 0 #Weight matrix for Moran's Is

    corMats = if(soCor=="Conditional"){
        lapply(seq_along(initialFit$lambda), function(lam){
            getMargCorMat(kernelData = cbind("a" = c(kernelData[[outVar]] - meanCenterVec(predict(initialFit, newx = as.matrix(kernelData[,xNames]), s = initialFit$lambda[lam]))), kernelData[, c("x", "y")]), glsSt = glsSt, control = optControl)
        })
        }else {NULL}
    #Manual loop over lambdas needed for gam
    fits = lapply(seq_len(nfolds), function(i){
        id = folds!=i
        if(soCor %in% c("GAM", "Conditional")){
            lapply(seq_along(lambdas), function(lam){
                fitPLSsingle(x = as.matrix(kernelData[id,xNames, drop = FALSE]),
                             y = kernelData[id, outVar], alpha = alpha, keep = keep,
                             lambda = lambdas[lam], soCor = soCor, xNames = xNames,
                             outVar = outVar, kernelData = kernelData[id,], glsSt = glsSt,
                             Coef = coef(glsSt), optControl = optControl)#corMats[[lam]]$Coef
            })
        } else{
        fitPLSsingle(x = as.matrix(kernelData[id,xNames, drop = FALSE]), y = kernelData[id, outVar], alpha = alpha, keep = keep, lambda = lambdas, soCor = soCor, xNames = xNames, outVar = outVar, kernelData = kernelData[id,], glsSt = glsSt, optControl = optControl)
        }
    })
    R2sCv = lapply(seq_len(nfolds), function(i){
        id = folds==i
        coefs = getCoefPenalised(fits[[i]], soCor, xNames = xNames)
        preds = apply(cbind(1, as.matrix(kernelData[id,xNames, drop = FALSE])) %*% coefs, 2, transFun)
        testDat = transFun(kernelData[id,outVar])
        R2 = 1-colMeans((preds-testDat)^2)/varN(testDat)
        moransI = apply((preds-testDat), 2, moransIobs, W = weightMat[id, id]) + 1/(nrow(preds)-1)
        return(cbind("R2" = R2, "moransI" = moransI))
    })
    #Use lambda within 1se from minimum error
    if(grepl("inla", soCor)){
        cvR2 = mean(sapply(R2sCv, function(x) x[,"R2"]))
        cvMoransI = mean(sapply(R2sCv, function(x) x[,"moransI"]))
        seId = lambdas = NA
    } else {
    cvEsts = rowMeans(R2s <- sapply(R2sCv, function(x) x[,"R2"]))
    maxId = which.max(cvEsts) #minimum location
    sdMax = sqrt(mean((R2s[maxId,] - cvEsts[maxId])^2)/(length(lambdas)-1))
    cvId = cvEsts > (cvEsts[maxId] - sdMax)
    seId = which.max(cvId)
    cvR2 = cvEsts[seId]
    cvMoransI = mean(sapply(R2sCv, function(x) x[seId,"moransI"]))
    }
    samData = if(finalFitFull) kernelData else kernelData[folds != sample(seq_len(nfolds), 1),]
    samData[,outVar] = meanCenterVec(samData[, outVar])
    finalFit = fitPLSsingle(x = samData[,xNames], y = samData[, outVar], xNames = xNames,
                            outVar = outVar, alpha = alpha, keep = keep, soCor = soCor, lambda = lambdas[seId],
                            kernelData = samData, glsSt = glsSt, optControl = optControl, Coef = corMats[[seId]]$Coef)
    return(list("fit" = finalFit, "cvR2" = cvR2, "lambdaMin" = lambdas[seId], "cvMoransI" = cvMoransI))
}




