#Extract coefficients
getCoef = function(fit){
    if(is.numeric(fit)){
        fit
    } else if (class(fit)[1] %in% c("esf", "resf")){
        fit$b[,"Estimate"]
    } else if(class(fit)[1]=="gam") {
        tmpCoef = coef(fit)
        tmpCoef[grep(names(tmpCoef), pattern = "te(x,y)", invert = TRUE, fixed = TRUE)] #Exclude spline components for gam for prediction
    } else if (tolower(class(fit))=="sarlm"){
        coef(fit)[-1]
    }else if (class(fit)=="summary.inla"){
        fit$fixed[, "mean"]
    }else if(inherits(fit, "try-error")){
        return(rep(NA, 2))
    } else {
        coef(fit)
    }
}
getCoefPenalised = function(fit, soCor, xNames = NULL, final = FALSE){
    if(soCor == "GAM"){
               Coef = if(final){
                 coef(fit$cv.glmnet)
               } else {
                   Reduce(f = cbind,sapply(fit, function(x) coef(x$cv.glmnet)))
               }
               rownames(Coef) = gsub("X", "", rownames(Coef))
               return(Coef)
           } else if(soCor == "Conditional"){
               Coef = (if(final){
                   coef(fit)
               } else {
                   Reduce(f = cbind, sapply(fit, function(x) coef(x)))
               })[-1,,drop = FALSE]
               rownames(Coef) = gsub("X", "", rownames(Coef))
               return(Coef)
           } else if(soCor %in% c("sf", "sfRandom")){
               tmpCoef = coef(fit)
               return(tmpCoef[c("(Intercept)", xNames),,drop = FALSE])
           } else if(soCor %in% c("inlaAR1", "inlaMatern")){
               fit$fixed[, "mean", drop = FALSE]
            } else {
                   return(coef(fit))
               }
}
#Extract standard errors for low-dimensional models
getSdCoef = function(fit, pattern = "PC"){
    cf = class(fit)[[1]]
    ses = if(cf == "lm"){
        summary(fit)$coef[-1, 2]
    } else if(cf=="gls"){
        sqrt(diag(summary(fit)$varBeta))[-1]
    } else if(cf=="gam"){
        summary(fit)$se[grep(names(summary(fit)$se), pattern = pattern)]
    }else if(cf == "resf"){
        fit$b$SE[-1]
    } else if (tolower(cf) == "sarlm"){
        summary(fit)$rest.se[-1]
    } else if(cf=="summary.inla"){
        fit$fixed[-1, "sd"]
        }
    names(ses) = paste0("PC", seq_along(ses))
    return(ses)
}
