# Get diagnostics of a high-dimensional fit
getDiagnostics = function(fit, kernelData, testData, testDataNone, soCor, transFun = meanCenterVec){
  weightMat = 1/as.matrix(dist(kernelData[, c("x", "y")])); diag(weightMat) = 0 #Weight matrix for Moran's I
  if(inherits(fit, "try-error")){
    return(c( "cvR2" = 0, "testR2" = 0, "testR2none" = 0, "Sens" = 0,
              "TDP" = NA, "MSEparam" = NA, "trueSignRate" = NA, "lambdaMin" = NA,
              "numNonZeroParams" = 0, "MeanAbsolute\nBiasParam" = NA,
              "moransIresid" = moransIobsLoc(kernelData$a, kernelData[, c("x", "y")]),
              "percSpatialCalled" = NA, "moransItest" = NA, "cvMoransI" = NA))
  }
  xNames = grep(names(kernelData), pattern = "b.", value = TRUE)
  Coef = getCoefPenalised(fit$fit, soCor, xNames = xNames, final = TRUE)
  truePreds = grep("TP", colnames(kernelData), value = TRUE)
  estPreds = grep(pattern = "Intercept", value = TRUE, invert = TRUE, rownames(Coef)[which(Coef!=0)])
  Sens = mean(truePreds %in% estPreds)
  TDP = if(length(estPreds)) mean(estPreds %in% truePreds) else NA
  MSEparam = mean((attr(kernelData, "betas") - Coef[-1])^2) #MSE of the beta parameter
  BiasParam = mean(abs((attr(kernelData, "betas") - Coef[-1])))#Bias of the beta parameter
  percSignRight = mean((sign(Coef[-1])==sign(attr(kernelData, "betas")))[Coef[-1]!=0])#True sign rate
  percSpatialCalled = mean(grepl(estPreds, pattern = "kernel"))
  moransIresid = ape::Moran.I(x = as.vector(kernelData$a - (cbind(1,as.matrix(kernelData[, xNames])) %*% Coef)), scaled = TRUE, weight = weightMat)$observed + 1/(length(kernelData$a)-1)#Find Moran's I on residuals

  predNew = transFun((cbind(1,as.matrix(testData[, grep(names(testData), pattern = "b.")])) %*% Coef)) #Prediction on new plot
  testR2 = 1-mean((testData$a-predNew)^2)/varN(testData$a)
  moransItest = moransIobsLoc((testData$a-predNew), testData[, c("x", "y")])+ 1/(length(predNew)-1)
  predNewNone = transFun((cbind(1,as.matrix(testDataNone[, grep(names(testDataNone), pattern = "b.")])) %*% Coef)) #Prediction on independent data
  testR2none = 1-mean((testDataNone$a-predNewNone)^2)/varN(testDataNone$a)
  return(c("cvR2" = unname(fit$cvR2), "testR2" = testR2, "testR2none" = testR2none, "Sens" = Sens,
           "TDP" = TDP, "MSEparam" = MSEparam, "trueSignRate" = percSignRight, "lambdaMin" = fit$lambdaMin,
    "numNonZeroParams" = sum(Coef[-1]!=0), "MeanAbsolute\nBiasParam" = BiasParam, "cvMoransI" = unname(fit$cvMoransI),
    "moransIresid" = moransIresid, "percSpatialCalled" = percSpatialCalled, "moransItest" = moransItest))
}