#Create folds for block cross-validation
makeBlockFolds = function(grid, nfolds, algo = "kmeans"){
    if(algo == "kmeans"){
        kmeans(grid, centers = nfolds)$cluster
    } else if(algo == "hclust"){
        hres = hclust(dist(grid))
        cutree(hres, k = nfolds)
    } else {
        stop("algorithm unknown")
    }
}
makeFolds = function(nfolds, data, cvType){
    folds = switch(cvType,
       "Random" = sample(nfolds, nrow(data), replace = TRUE),#Also uneven fold sizes
       "Blocked" = makeBlockFolds(data[, c("x", "y")], nfolds),
       stop("Unknown CV strategy"))
    while(any(table(folds)<=1)){
        folds = makeFolds(nfolds, data, cvType)
    }
    return(folds)
}
#Set buffer observations to NA
bufferFolds = function(folds, testFoldId, grid, quant = 1){
    withinTestDist = quantile((dist2point(grid[testFoldId==folds,], colMeans(grid[testFoldId==folds,]))), quant)#max distance to center
    dists2TestFold = apply(grid[testFoldId==folds,], 1, function(x) dist2point(grid[testFoldId!=folds,], x))
    minDists2TestFold = apply(dists2TestFold, 1, min)
    folds[testFoldId!=folds][minDists2TestFold<withinTestDist] = NA
    return(folds)
}