#' Build a spatial dataframe
#'
#' @param depTypeResp,deptTypeReg spatial dependence types for outcome and regressors
#' @param gridSize
#' @param n the sample size
#' @param grid
#' @param p the number of regressors
#' @param betas the parameters
#' @param sdResp,sdReg (residual) standard deviation of outcome and regressor
makeSpatialDf = function(depTypeResp = c("none", "kernel", "linear", "edge", "gaussCor", "spherCor", "linCor", "linear_gaussCor",
                                         "edge_gaussCor"),
                         depTypeReg = c("none", "kernel", "linear", "edge", "gaussCor", "spherCor", "linCor", "linear_gaussCor",
                                        "edge_gaussCor"),
                         gridSize = nrow(grid), n, grid, p = length(betas), csList, subGrid = NULL,
                         betas = integer(1), sdResp = 1, sdReg = 1, scale = TRUE, center = TRUE){
    depTypeResp = match.arg(depTypeResp); depTypeReg = match.arg(depTypeReg)
    if(is.null(subGrid)){
        gridId = sample(gridSize, n)
        subGrid = grid[gridId,]
    }
    correlationMatResp = getCorMat(subGrid, depTypeResp, csList)
    correlationMatReg = getCorMat(subGrid, depTypeReg, csList)
    xyBetasResp = xyBetasReg = NULL
    b  = t(rmvnorm(1, drawKernel(gridSize, grid = subGrid, depType = depTypeReg, xyBetas = xyBetasReg, maxGrid = apply(grid, 2, max)), sigma = sdReg *correlationMatReg))
    #b = scale(b, center = center, scale = scale) #No scaling
    a  = t(rmvnorm(1, drawKernel(gridSize, grid = subGrid, depType = depTypeResp, xyBetas = xyBetasResp, maxGrid = apply(grid, 2, max)) + b %*% betas,
               sigma = correlationMatResp*sdResp))
    a = (a-mean(a))#/sd(a)
    cbind(subGrid, "a" = a, "b" = b)
}
