#Helper functions for univariate fitters
getUniFits = function(df, corStructList, glsControl, kSpline = 5, kSF = 8, rowColCorrect = FALSE, inlaIn = FALSE){
    Form = if(rowColCorrect) "a~b+row+col" else "a~b"
    olsFit = lm(Form, data = df)
    glsFits = lapply(corStructList, function(corStruct){
        try(gls(formula(Form), correlation = corStruct, data = df, control  = glsControl), silent = TRUE)
    })
    glsFitsMarginal = lapply(corStructList, function(corStruct){
        margCorMat = getMargCorMat(kernelData = df, glsSt = glsStruct(corStruct),
                                   control = optControl, outVar = "a")$corMat
        desMat = data.frame(margCorMat %*% as.matrix(df[, c("b", "a")]))
        colnames(desMat) = c("b", "a")
        lm(a~b, data = desMat)
    })
    names(glsFits) = paste0("gls_", names(csList))
    names(glsFitsMarginal) = paste0("glsMarg_", names(csList))
    gamFit = gam(formula(paste(Form, "+ te(x,y, k = kSpline, bs = c('tp','tp'))")), data = df)
    sfFit = try(getSFfit(df, k = kSF, Form =  Form), silent = TRUE)
    Regs = if(rowColCorrect) c("b", "row", "col") else "b"
    sfFitMoran = getSFfitspmoran(df, regs = Regs)
    sfFitMoranRandom = getSFfitspmoran(df, random = TRUE, regs = Regs)
    sar = try(fitSAR(df, neighbour = "distance", regs = Regs), silent = TRUE)
    sarknn = try(fitSAR(df, neighbour = "knn", regs = Regs), silent = TRUE)
    if(inlaIn){
        inlaAR1 = try(fitInla(df, "AR1"), silent = TRUE)
        inlaMatern = try(fitInla(df, "Matern"), silent = TRUE)
    }
    c(list("ols" = olsFit, "gam" = gamFit, "sar" = sar, "sarknn" = sarknn,
           "sfMoran" = sfFitMoran, "sfMoranRandom" = sfFitMoranRandom), if(inlaIn) list("inlaAR1" = inlaAR1, "inlaMatern" = inlaMatern) else NULL,  glsFits, glsFitsMarginal)
}
getUniPs = function(fit){
    switch(tolower(class(fit)[1]),
           "lm" = summary(fit)$coef["b", 4],
           "gam" = summary(fit)$p.table["b",4],
           "gls" = anova(fit)["b", 3],
           "resf" = fit$b[2, 4],
           "esf" = fit$b[2, 4],
           "sarlm" = summary(fit)$Coef["b", 4],
           "summary.inla" =  if(sign(fit$fixed["b", "0.025quant"])==sign(fit$fixed["b", "0.975quant"])) 0 else 1,
           "try-error" = NA,
           stop("Unknown class!")
    )
}
